<!-- templates/article_delete.html -->
{% extends 'opcalendar/base.html' %}

{% block title %}
Event Delete
{% endblock %}

{% block content %}
    <h1>Delete</h1>
    <form action="" method="post">{% csrf_token %}
        <p>Are you sure you want to delete this event?</p>
        <button class="btn btn-danger ml-2" type="submit">Confirm</button>
    </form>
{% endblock content %}
