# Generated by Django 3.1.7 on 2021-04-29 11:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0023_auto_20210330_0632"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="ingameevents",
            name="moon_extraction",
        ),
        migrations.AddField(
            model_name="eventcategory",
            name="description",
            field=models.TextField(
                blank=True,
                help_text="Prefilled description that will be added on default on the event description.",
            ),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="is_default",
            field=models.BooleanField(
                default=False,
                help_text="Whether this host is used as the default value on the event form",
            ),
        ),
        migrations.AddField(
            model_name="eventvisibility",
            name="include_in_feed",
            field=models.BooleanField(
                default=False,
                help_text="Whether these events should be included in the ical feed.",
            ),
        ),
        migrations.AddField(
            model_name="eventvisibility",
            name="is_default",
            field=models.BooleanField(
                default=False,
                help_text="Whether this visibility filter is used as the default value on the event form",
            ),
        ),
        migrations.AddField(
            model_name="eventvisibility",
            name="is_visible",
            field=models.BooleanField(
                default=True,
                help_text="Whether this visibility filter should be displayed on the event form. Disable for internal visibilities such as the NPSI import fleet visibilities.",
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="source",
            field=models.CharField(
                choices=[
                    ("LN", "EVE LinkNet"),
                    ("SF", "Spectre Fleet"),
                    ("EU", "EVE University"),
                    ("FI", "Fun Inc."),
                    ("FY", "FRIDAY YARRRR"),
                    ("RR", "Redemption Road"),
                    ("CA", "CAS"),
                    ("FD", "Fwaming Dwagons"),
                    ("FR", "FREE RANGE CHIKUNS"),
                ],
                help_text="The API source where you want to pull events from",
                max_length=32,
            ),
        ),
    ]
